% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %                                                                       %
% %    Matlab script for lecture                                          %
% %    "Simple differential equations - introduction using Matlab"        %
% %    by Christian Rummel                                                %
% %                                                                       %
% %    BENESCO lecture series                                             %
% %    "Signal Analysis and Brain Oscillations in Health and Disease"     %
% %    Bern, 2017/11/24                                                   %
% %                                                                       %
% %    Script demonstrates integration of two coupled FitzHugh-Nagumo     %
% %    ODEs. Coupling can be uni- and bidirectional.                      %
% %                                                                       %
% %    written by C. Rummel, 2017/11/20                                   %
% %                                                                       %
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
close all
clc

% parameters of the integrator

d       =     4              % dimension of time series
N       = 20000              % length of time series
dt      =     0.1            % integration step

% parameters for display

n_min   =   -10;
n_max   =     0;
epsl_0  =     0.01;

N_ens   =    10;

% define parameters for usage also in function for derivative 

global alpha epsilon gamma v_eq

% alpha   = -0.139 * [0.8; -1.2];       % 1st neuron spikes, 2nd decays
alpha   = -0.139 * [0.8;  1.2];       % both neurons spike
epsilon =  0.008 * [0.8;  1.2];
gamma   =  1.5   * [0.8;  1.2];
v_eq    =          [0;    0];

% loop: coupling is increased on linearly or exponentially

global coupl                    % 2D matrix

epsl   = zeros(1,n_max-n_min+1);
corr_x = zeros(N_ens,n_max-n_min+1);

for n_cpl = n_min:n_max

%     epsl(n_cpl-n_min+1) = n_cpl*epsl_0;         % linear dependence
    epsl(n_cpl-n_min+1) = 2^n_cpl;              % exponential dependence
    coupl               = [0 epsl(n_cpl-n_min+1); 0 0];   % 1st neuron entrains 2nd neuron
%     coupl               = [0 0; epsl(n_cpl-n_min+1) 0];   % 2nd neuron entrains 1st neuron

% loop over independent realizations

    for n_ens = 1:N_ens
    
% use random initial condition and integrate FitzHugh-Nagumo system

%         x_0    = [0 -0.07 0 -0.07];
        x_0    = 0.5*rand(1,d);
        [x, t] = RungeKutta4 (@FitzHughNagumo2, d, N, dt, x_0);
        
% calculate Pearson correlation as interaction measure from second half of signals
        
        corr_x(n_ens,n_cpl-n_min+1) = corr (x(1,ceil(N/2):N)',x(3,ceil(N/2):N)');

    end

% display attractors in 2D

    figure(30+n_cpl)
    subplot(1,2,1);
    plot (x(1,:),x(2,:),'b');
    subplot(1,2,2);
    plot (x(3,:),x(4,:),'r');

% display time series of v- and w-coordinate

    figure(130+n_cpl)
    for i = 1:d/2
        subplot(d/2,1,i);
        hold on
        plot(t,x(i,:),'b');
        plot(t,x(i+d/2,:),'r');
        if i==1
            title(strcat('coupling = ',num2str(epsl(n_cpl-n_min+1))));
        end
    end
    
%     pause
    
end

% display correlation coefficient

figure(100)
errorbar(n_min:n_max,mean(corr_x),std(corr_x));



