% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %                                                                            %
% %   function [x, t] = RungeKutta4 (deriv, d, N, dt, x_0);                    %
% %                                                                            %
% %   INPUTS:                                                                  %
% %                                                                            %
% %   @deriv    specifies the script of the derivative, e.g. @LorenzODE_B      %
% %   d         dimension                                                      %
% %   N         number of datapoints                                           %
% %   dt        time step                                                      %
% %   x_0       initial conditions (column vector of dimension d)              %
% %                                                                            %
% %   OUTPUTS:                                                                 %
% %                                                                            %
% %   x         function  (matrix of dimensions d,N)                           %
% %   t         time line (row vector of dimension N)                          %
% %                                                                            %
% %   adapted from a code by R.G. Andrzejak, UPF, Barcelona                    %
% %                                                                            %
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [x, t] = RungeKutta4 (deriv, d, N, dt, x_0)

% define time line

    t      = 0:dt:(N-1)*dt;
    
% allocate function values and initialize    
    
    x      = zeros(d,N);
    x(:,1) = x_0;

% loop over all time points
    
    for n = 2:N
        k1     = dt*feval(deriv,x(:,n-1));
        k2     = dt*feval(deriv,x(:,n-1)+k1/2);
        k3     = dt*feval(deriv,x(:,n-1)+k2/2);
        k4     = dt*feval(deriv,x(:,n-1)+k3);
        
        x(:,n) = x(:,n-1) + k1/6 + k2/3 + k3/3 + k4/6;
    end    

end

